package com.starquest.kafka.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Properties;

import javax.xml.parsers.ParserConfigurationException;

import kafka.utils.ShutdownableThread;

import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import com.starquest.udb.kafka.JsonRowParser;
import com.starquest.udb.kafka.RowParser;
import com.starquest.udb.kafka.XmlRowParser;

public class Consumer extends ShutdownableThread
{
  private final KafkaConsumer<Integer, String> consumer;
  private final String topic;
  private RowParser parser;
  static Logger logger = Logger.getLogger(Consumer.class.getName());

  public Consumer(String propFile, String topic) throws ParserConfigurationException, SAXException, IOException
  {
    super("SQDR KafkaConsumerExample", false);
    Properties props = new Properties();
    props.put(ConsumerConfig.GROUP_ID_CONFIG, "DemoConsumer");
    props.put(ConsumerConfig.ENABLE_AUTO_COMMIT_CONFIG, "true");
    props.put(ConsumerConfig.AUTO_COMMIT_INTERVAL_MS_CONFIG, "1000");
    props.put(ConsumerConfig.SESSION_TIMEOUT_MS_CONFIG, "30000");
    props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.IntegerDeserializer");
    props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");
    
	Properties properties = new Properties();
	try {
		File propertiesFile = new File(propFile);
		// Write out properties a default properties file if it appears like it doesn't exist
		if (propertiesFile.exists() == false) {
			properties.setProperty(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, "172.19.36.175:6667");
		
			try {
				FileOutputStream outStream = new FileOutputStream(propertiesFile);
				properties.store(outStream, "# Kafka Properties\n\n");
				outStream.flush();
				outStream.close();
			} catch (FileNotFoundException e1) {
			} catch (IOException e2) {
			}
			System.exit(0);
		}
		FileInputStream inStream = new FileInputStream(propertiesFile);
		properties.load(inStream);
		Enumeration<?> e = properties.propertyNames();
		while (e.hasMoreElements()) {
			String propertyname = (String) e.nextElement();
			props.setProperty(propertyname, properties.getProperty(propertyname));
		}    				

    	
    } finally {
    	
    }

    consumer = new KafkaConsumer<>(props);
    this.topic = topic;
  }

  @Override
  public void doWork() {
	Collection<String> what = Collections.singletonList(this.topic);
    consumer.subscribe(what);
    ConsumerRecords<Integer, String> records = consumer.poll(1000);
    for (ConsumerRecord<Integer, String> record : records) {
    	try {
    		String val = record.value();
    		if (val.startsWith("<")) {
    			System.out.println("Pure XML: " + val);
    			if (parser == null || parser instanceof JsonRowParser)
    				parser = new XmlRowParser();
    		} else {
    			System.out.println("Raw JSON: " + val);
    			if (parser == null || parser instanceof XmlRowParser)
    				parser = new JsonRowParser();
    		}

			parser.parse(val);
			//<row> <column1=xsi:nil="true"/><column2="value"/></row>
		    System.out.println("Transaction: " + parser.getTransaction());
		    System.out.println("Operation: " + parser.getOperation());
		    System.out.println("Sequence: " + parser.getSequence());
		    System.out.println("Before Key: ");
	
			for (String key : parser.getBeforeKeys().keySet()) {
			      System.out.println(key + ": " + parser.getBeforeKeys().get(key));			      
		    }

			System.out.println("Name: " + parser.getDestinationName());
			
			System.out.println("row data: ");
	      
			for (String key : parser.getAfterImages().keySet()) {
			      System.out.println(key + ": " + parser.getAfterImages().get(key));			      
			}
    	} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
    		
    	}
    }
  }

  @Override
  public String name() {
    return null;
  }

  @Override
  public boolean isInterruptible() {
    return false;
  }	
  
  public static void main(String[] args) {
		BasicConfigurator.configure();
		logger.setLevel(Level.ERROR);
		Logger.getRootLogger().setLevel(Level.ERROR);
		if (args.length < 2) {
			System.out.println("Arguments: properties-file topic-name");
			return;
		}
		try {
			Consumer consumer = new Consumer(args[0], args[1]);
			consumer.start();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		

	}

}
